package com.frontleaves.bungeecross.configuration;

import com.frontleaves.bungeecross.BungeeCross;
import lombok.RequiredArgsConstructor;
import net.md_5.bungee.api.config.ServerInfo;
import net.md_5.bungee.config.Configuration;
import net.md_5.bungee.config.ConfigurationProvider;
import net.md_5.bungee.config.YamlConfiguration;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;

/**
 * 服务器启动执行操作
 *
 * @author 筱锋xiao_lfeng
 */
@RequiredArgsConstructor
public class ServerEnable {
    private final BungeeCross plugin;

    /**
     * 服务器启动内容
     */
    public void serverStart() {
        this.getConfig();
        this.getLang();
        this.getProxyServer();

        System.out.println(VariableStorage.getPrefix() + "§c配置文件加载完毕");
        System.out.println("§9§l§m=]=======§2 Bungee§aCross §9§l§m========[=");
        System.out.println("§6作者：§e" + BungeeCross.getAUTHOR());
        System.out.println("§9版本：§d" + BungeeCross.getVERSION());
        System.out.println("§9§l§m=]=======§2 Bungee§aCross §9§l§m========[=");
    }

    private void getLang() {

    }

    /**
     * 获取配置文件，如果没有则自动创建
     */
    private void checkConfigFile() {
        if (!plugin.getDataFolder().exists()) {
            if (plugin.getDataFolder().mkdir()) {
                plugin.getLogger().info("文件夹创建成功");
            } else {
                plugin.getLogger().info("文件夹创建失败");
            }
        }
        File configFile = new File(plugin.getDataFolder(), "config.yml");
        if (!configFile.exists()) {
            saveDefaultConfigYmlFile(configFile);
            plugin.getLogger().info("配置文件创建成功");
        }
    }

    /**
     * 获取配置文件
     */
    private void getConfig() {
        this.checkConfigFile();
        Configuration config;
        File configFile;
        // 读取文件信息
        try {
            configFile = new File(plugin.getDataFolder(), "config.yml");
            config = ConfigurationProvider.getProvider(YamlConfiguration.class).load(configFile);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 读取配置文件
        if (config.getString("bungee-cord.server-prefix") != null && !config.getString("bungee-cord.server-prefix").isEmpty()) {
            VariableStorage.setPrefix(config.getString("bungee-cord.server-prefix"));
        } else {
            VariableStorage.setPrefix("§7[§2Bungee§aCross§7] §r");
        }
        // 读取登录服务器
        if (config.getList("bungee-cord.server-login").isEmpty()) {
            this.reloadTheDefaultConfig(configFile);
            return;
        } else {
            VariableStorage.setLoginServer(config.getList("bungee-cord.server-login"));
        }
        // 读取大厅服务器
        if (config.getList("bungee-cord.server-lobby").isEmpty()) {
            this.reloadTheDefaultConfig(configFile);
            return;
        } else {
            VariableStorage.setLobbyServer(config.getList("bungee-cord.server-login"));
        }
        // 获取登陆提醒
        if (config.getString("bungee-cord.login-reminder") != null && !config.getString("bungee-cord.login-reminder").isEmpty()) {
            VariableStorage.setBungeeCordReminder(config.getBoolean("bungee-cord.login-reminder"));
            if (config.getString("bungee-cord.join-message") != null && !config.getString("bungee-cord.join-message").isEmpty()) {
                VariableStorage.setJoinMessage(config.getString("bungee-cord.join-message"));
            } else {
                VariableStorage.setJoinMessage("§7[§a+§7] §6%player%");
            }
            if (config.getString("bungee-cord.leave-message") != null && !config.getString("bungee-cord.leave-message").isEmpty()) {
                VariableStorage.setLeaveMessage(config.getString("bungee-cord.leave-message"));
            } else {
                VariableStorage.setLeaveMessage("§7[§c-§7] §6%player%");
            }
        } else {
            config.set("bungee-cord.login-reminder", true);
            VariableStorage.setBungeeCordReminder(true);
        }
        // 是否开启管理员静默登录
        if (config.getString("bungee-cord.login-silent") != null && !config.getString("bungee-cord.login-silent").isEmpty()) {
            VariableStorage.setBungeeSilent(config.getBoolean("bungee-cord.login-silent"));
        } else {
            config.set("bungee-cord.login-silent", true);
            VariableStorage.setBungeeSilent(true);
        }

        // 获取服务器列表

        plugin.getProxy().getServers().forEach((string, serverInfo) -> {
            if (config.getSection("server").getKeys().contains(string)) {
                if (config.getString("server." + string + ".display-name") == null && config.getString("server." + string + ".display-name").isEmpty()) {
                    this.reloadTheDefaultConfig(configFile);
                    return;
                }
                if (config.getString("server." + string + ".bungee-join-server-reminder") == null && config.getString("server." + string + ".bungee-join-server-reminder").isEmpty()) {
                    this.reloadTheDefaultConfig(configFile);
                    return;
                }
                if (config.getString("server." + string + ".join-server-inform.join-enable") == null && config.getString("server." + string + ".join-server-inform.join-enable").isEmpty()) {
                    this.reloadTheDefaultConfig(configFile);
                    return;
                }
                if (config.getString("server." + string + ".join-server-inform.leave-enable") == null && config.getString("server." + string + ".join-server-inform.leave-enable").isEmpty()) {
                    this.reloadTheDefaultConfig(configFile);
                    return;
                }
                if (config.getString("server." + string + ".join-server-inform.join-message") == null && config.getString("server." + string + ".join-server-inform.join-message").isEmpty()) {
                    this.reloadTheDefaultConfig(configFile);
                    return;
                }
                if (config.getString("server." + string + ".join-server-inform.leave-message") == null && config.getString("server." + string + ".join-server-inform.leave-message").isEmpty()) {
                    this.reloadTheDefaultConfig(configFile);
                }
            } else {
                config.set("server." + serverInfo.getName() + ".display-name", serverInfo.getName());
                config.set("server." + serverInfo.getName() + ".bungee-join-server-reminder", true);
                config.set("server." + serverInfo.getName() + ".join-server-inform.join-enable", true);
                config.set("server." + serverInfo.getName() + ".join-server-inform.leave-enable", true);
                config.set("server." + serverInfo.getName() + ".join-server-inform.join-message", "[+] %player% 加入到 %server_display% 服务器");
                config.set("server." + serverInfo.getName() + ".join-server-inform.leave-message", "[-] %player% 加入到 %server_display% 服务器");
            }
        });
        try {
            ConfigurationProvider.getProvider(YamlConfiguration.class).save(config, configFile);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    private boolean checkServerNameCorrect(String serverName) {
        ServerInfo serverInfo = plugin.getProxy().getServerInfo(serverName);
        return serverInfo != null;
    }

    /**
     * 获取全部服务器
     */
    private void getProxyServer() {
        VariableStorage.setProxyServer(plugin.getProxy().getServers());
    }

    /**
     * 复制默认配置文件到配置项
     *
     * @param configFile File
     */
    private void saveDefaultConfigYmlFile(@NotNull File configFile) {
        // 将默认配置文件从资源中复制到插件目录
        try (InputStream inputStream = plugin.getResourceAsStream("config.yml")) {
            Files.copy(inputStream, configFile.toPath());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
    
    private void saveDefaultLangYmlFile(@NotNull File langFile) {
        // 将默认配置文件从资源中复制到插件目录
        try (InputStream inputStream = plugin.getResourceAsStream("lang.yml")) {
            Files.copy(inputStream, langFile.toPath());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 重新载入新的配置文件
     *
     * @param configFile File
     */
    private void reloadTheDefaultConfig(@NotNull File configFile) {
        File newConfigFile = new File(plugin.getDataFolder(), "config.old.yml");
        if (newConfigFile.exists()) {
            newConfigFile.delete();
        }
        try {
            Files.copy(configFile.toPath(), newConfigFile.toPath());
            if (configFile.delete()) {
                this.saveDefaultConfigYmlFile(configFile);
                plugin.getLogger().warning("配置文件错误，已重新生成新的文件");
                plugin.getLogger().info("旧配置文件已生成 config.old.yml");
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
